var
	chai 	= require('chai'),
	assert	= chai.assert,
	expect  = chai.expect,
	fs 		= require('fs'),
	logger 	= require('./index.js');


function checkFile(fileName, cb){
	setTimeout(function(){
		if(fs.existsSync(fileName) && (contents = fs.readFileSync(fileName, 'utf8'))){
			fs.unlinkSync(fileName);
			cb(contents);
		}
		else {
			cb(false);
		}
	}, 10);
}


describe('Checking File Writer', function(){

	var z = { 'a': 'b' };

	it('Writing to Default File', function(done){
		var L = new logger();
		L.log('OK', z);

		checkFile('default.access.log', function(c){
			assert.match(c, /^(info )([0-9\-\s\.\:\[\]]+)(OK)(.*)\n/, 'Logs Not Matching');
			done();
		});

	})


	it('Writing to New File', function(done){
		var L = new logger();
		L.setOut('default1.log');
		L.log('OK', z);

		checkFile('default1.log', function(c){
			assert.match(c, /^(info )([0-9\-\s\.\:\[\]]+)(OK)(.*)\n/, 'Logs Not Matching');
			done();
		});
	})

	it('Writing to Existing File', function(done){
		var L = new logger();
		fs.closeSync(fs.openSync('default2.log', 'w'));
		L.setOut('default2.log');

		L.log('OK', z);

		checkFile('default2.log', function(c){
			assert.match(c, /^(info )([0-9\-\s\.\:\[\]]+)(OK)(.*)\n/, 'Logs Not Matching');
			done();
		});
	})


	it('Error on Writing to not accessable content', function(done){
		var L = new logger();
		L.setOut('/var/nt.log');
	
		L.log('OK', z);
		done();
	})

})


